==> GeneticsBase_1.0.0.tar.gz: GeneticsBase/R/makeCCodesAndTables.R <==
##
## Utility functions
##

msubstring <- function(str, sepv)
  {
    startvec <- c(1,sepv+1)
    endvec   <- c(sepv,1e6)
    lapply(as.character(str), substring, first=startvec, last=endvec)
  }

lastChar <- function(str)
  {
    l <- nchar(str)
    substr(str,l,l)
  }


dropLastChar <- function(str)
  {
    l <- nchar(str)
    substr(str,1,l-1)
  }


commonStr <- function(x)
  {
    x <- as.character(x)
    endCommon <- -1
    for(i in 1:min(nchar(x)))
      if(all(
             substr(x[1],1,i)==substr(x,1,i)
             )
         )
        endCommon <- i
    if(endCommon<1)
      {
        warning("No common substring. Returning first element.")
        endCommon <- nchar(x[1])
      }
    retval <- substring(x[1],1,endCommon)
    retval
  }



##
## The real code
## 
makeCCodesAndTTables <- function(
                                 data,
                                 format=c("single",
                                          "adjacent"),
                                 ploidy=2,
                                 alleles=NULL,
                                 sep="/",
                                 remove.spaces=TRUE,
                                 reorder=c("freq", "yes", "no",
                                           "default", "ascii"),
                                 allow.partial.missing=FALSE,
                                 markerNames,
                                 ...
                                 )
{
  ## procues arguments ##
  format  <- match.arg(format)
  reorder <- match.arg(reorder)
  
  ## Step 0: Create an nObservation * nmarker * nallele *  array 

  nmarker <- switch(format,
                    "single"=ncol(data),
                    "adjacent"=ncol(data)/ploidy
                    )
  if(format=="adjacent columns" && round(marker)!=marker)
    stop("Specified ploidy (", ploidy,
         ") is incompatible with number of columns (", ncol(data) ,")",
         sep="")
  nObs <- nrow(data)
  
  alleleArray <- array(as.character(NA),
                       dim=c(nObs, nmarker, ploidy)
                       )

  obsNames <- rownames(data)
  
  if(format=="single")
    {
      if(is.character(sep))
        {
          ## one list per marker, one list per allele
          tmp <- lapply(data, function(x) strsplit(as.character(x), sep))
        }
      else if (is.numeric(sep))
        {
          if(length(sep)!=ploidy-1)
            stop("Invalid value for `sep'. When using positional separation (numeric `sep'), `sep' must have length ploidy-1.")
          
          tmp <- lapply(data, msubstring, sep)
          
        }
      else
        stop("Invalid value for `sep'.  It should be a character or numeric vector of separator markers/positions")

      makelen <- function(x, ploidy) 
        c(x, rep(NA, max(0,ploidy-length(x))))
        
      ## add NA to fill empty positions, and retrun as a matrix
      tmp <- lapply(tmp, function(x) t(sapply(x, makelen, ploidy=2)))
      ## tmp is now one matrix per marker, one column per allele

      if(missing(markerNames))
        markerNames <- colnames(data)

      ## put into the array
      for(i in 1:length(tmp))
        alleleArray[,i,] <- tmp[[i]]
    }
  else # format=="adjacent"
    {
      dIndex <- 1
      for(i in 1:nmarker)
        for(j in 1:ploidy)
          {
            alleleArray[,i,j] <- data[,dIndex]
            dIndex <- dIndex+1
          }
      
      if(missing(markerNames))
        {

          ## Extract marker names as common substring in columns
          ## corresponding to a # single marker
          cNames <- split(colnames(data), sort(rep(1:nmarker, ploidy)), )
          markerNames <- sapply( cNames, commonStr )
          if(all(lastChar(markerNames)=="."))
            markerNames <- dropLastChar(markerNames)
        }
    }

  reorder.fun  <- function( x, alleles)
    {
      tmp <- match( x, alleles )
      x[order(tmp)]
    }
      
    
  if(reorder=="ascii" || reorder=="no")
    ## compute order of alleles, most common to least common,  for use below
    alleleTable <- apply( alleleArray, 2:3, function(x) sort(unique(x)))
  else 
    alleleTable <- apply( alleleArray, 2:3,
                       function(x) names(sort(table(x), decreasing=TRUE)))

  # convert alleleTab back into a list
  attr(alleleTable,"dim") <- NULL
    

  if(reorder!="no") # reorder alleles within observations
    {
      for(i in 1:nObs)
        for(j in 1:nmarker)
          alleleArray[i,j,] <- reorder.fun(alleleArray[i,j,], alleleTable[[j]])
    }

  ## Join the alleles back up into a single slash-separated string
  alleleMat <- alleleArray[,,1]
  for(i in 2:ploidy)
    {
      if(allow.partial.missing)
        ind <- TRUE
      else
        ind <- !is.na(alleleArray[,,i])
      alleleMat[ind] <- paste(alleleMat[ind], alleleArray[,,i][ind], sep="/")
    }

  transTables <- vector(mode="list",length=nmarker)
  for(i in 1:nmarker)
    {
      levs <- !duplicated(alleleMat[,i])

      transTables[[i]] <- cbind(
                              alleleMat[levs,i],
                              alleleArray[levs,i,]
                              )
      colnames(transTables[[i]]) <- c("levels",
                                      paste("allele",1:ploidy,sep="."))
                              
    }
  names(transTables) <- markerNames


  callCodes <- matrix(as.integer(NA),
                      nrow=ncol(alleleMat),
                      ncol=nrow(alleleMat)
                      )
  rownames(callCodes) <- markerNames
  colnames(callCodes) <- rownames(data)

  for(i in 1:nmarker)
    {
      callCodes[i,] <- match(alleleMat[,i], transTables[[i]][,1])
    }
  
  retval <- list(
                 callCodes=callCodes,
                 transTables=transTables
                 )
  
  retval
  
}

==> GeneticsBase_1.14.0.tar.gz: GeneticsBase/R/makeCCodesAndTables.R <==

##
## Utility functions
##

msubstring <- function(str, sepv)
  {
    startvec <- c(1,sepv+1)
    endvec   <- c(sepv,1e6)
    lapply(as.character(str), substring, first=startvec, last=endvec)
  }

lastChar <- function(str)
  {
    l <- nchar(str)
    substr(str,l,l)
  }


dropLastChar <- function(str)
  {
    l <- nchar(str)
    substr(str,1,l-1)
  }


commonStr <- function(x)
  {
    x <- as.character(x)
    endCommon <- -1
    for(i in 1:min(nchar(x)))
      if(all(
             substr(x[1],1,i)==substr(x,1,i)
             )
         )
        endCommon <- i
    if(endCommon<1)
      {
        warning("No common substring. Returning first element.")
        endCommon <- nchar(x[1])
      }
    retval <- substring(x[1],1,endCommon)
    retval
  }



##
## The real code
## 
makeCCodesAndTTables <- function(
                                 data,
                                 format=c("single",
                                          "adjacent"),
                                 ploidy=2,
                                 alleles=NULL,
                                 sep="/",
                                 remove.spaces=TRUE,
                                 reorder=c("freq", "yes", "no",
                                           "default", "ascii"),
                                 allow.partial.missing=FALSE,
                                 markerNames,
                                 ...
                                 )
{
  ## procees arguments ##
  format  <- match.arg(format)
  reorder <- match.arg(reorder)

  ## TODO: Fix handling of alleles
  if(!missing(alleles) && !is.null(alleles) && !is.na(alleles))
    stop("Argument 'alleles' not yet handled.")
  

  
  if(!is.data.frame(data))
    data <- as.data.frame(data)

  if(remove.spaces)
    for(i in 1:ncol(data))
      {
        data[,i] <- gdata::trim(as.character(data[,i]))
      }
  
  
  nMarker <- switch(format,
                    "single"=ncol(data),
                    "adjacent"=ncol(data)/ploidy
                    )
  if(format=="adjacent columns" && round(nMarker)!=nMarker)
    stop("Specified ploidy (", ploidy,
         ") is incompatible with number of columns (", ncol(data) ,")",
         sep="")
  nObs <- nrow(data)


  
  alleleArray <- array(as.character(NA),
                       dim=c(nObs, nMarker, ploidy)
                       )

  obsNames <- rownames(data)
  
  if(format=="single")
    {
      if(is.character(sep))
        {
          ## one list per marker, one list per allele
          tmp <- lapply(data, function(x) strsplit(as.character(x), sep))
        }
      else if (is.numeric(sep))
        {
          if(length(sep)!=ploidy-1)
            stop("Invalid value for `sep'. When using positional separation (numeric `sep'), `sep' must have length ploidy-1.")
          
          tmp <- lapply(data, msubstring, sep)
          
        }
      else
        stop("Invalid value for `sep'.  It should be a character or numeric vector of separator markers/positions")

      makelen <- function(x, ploidy) 
        c(x, rep(NA, max(0,ploidy-length(x))))
        
      ## add NA to fill empty positions, and retrun as a matrix
      tmp <- lapply(tmp, function(x) t(sapply(x, makelen, ploidy=2)))
      ## tmp is now one matrix per marker, one column per allele

      if(missing(markerNames))
        markerNames <- colnames(data)

      ## put into the array
      for(i in 1:length(tmp))
        alleleArray[,i,] <- tmp[[i]]
    }
  else # format=="adjacent"
    {
      dIndex <- 1
      for(i in 1:nMarker)
        for(j in 1:ploidy)
          {
            alleleArray[,i,j] <- data[,dIndex]
            dIndex <- dIndex+1
          }
      
      if(missing(markerNames))
        {

          ## Extract marker names as common substring in columns
          ## corresponding to a # single marker
          cNames <- split(colnames(data), sort(rep(1:nMarker, ploidy)), )
          markerNames <- sapply( cNames, commonStr )
          if(all(lastChar(markerNames)=="."))
            markerNames <- dropLastChar(markerNames)
        }
    }

  reorder.fun  <- function( x, alleles)
    {
      tmp <- match( x, alleles )
      x[order(tmp)]
    }

  if(reorder=="yes")
    stop("'reorder=\"yes\"' is not yet supported.")
  else if(reorder=="ascii" || reorder=="no")
    ## compute order of alleles, most common to least common,  for use below
    alleleTable <- apply( alleleArray, 2:3, function(x) sort(unique(x)))
  else 
    alleleTable <- apply( alleleArray, 2:3,
                       function(x) names(sort(table(x), decreasing=TRUE)))

  ## convert alleleTab back into a list
  attr(alleleTable,"dim") <- NULL
    

  if(reorder!="no") # reorder alleles within observations
    {
      for(i in 1:nObs)
        for(j in 1:nMarker)
          alleleArray[i,j,] <- reorder.fun(alleleArray[i,j,], alleleTable[[j]])
    }

  ## Convert empty alleles to missing values 
  alleleArray[ alleleArray <= "" ] <- NA  # convert empty alleles to missing
   
  ## Join the alleles back up into a single slash-separated string
  alleleMat <- alleleArray[,,1]
  dim(alleleMat) <- dim(alleleArray)[-3]
  for(i in 2:ploidy)
    {
      if(allow.partial.missing)
        ind <- TRUE
      else
        ind <- !is.na(alleleArray[,,i,drop=FALSE])
      alleleMat[ind] <- paste(alleleMat[ind],
                              alleleArray[,,i,drop=FALSE][ind],
                              sep="/")
    }

  transTables <- vector(mode="list",length=nMarker)
  for(i in 1:nMarker)
    { 
      levs <- !duplicated(alleleMat[,i])
      if(sum(levs)>1)
      transTables[[i]] <- cbind(
                              alleleMat[levs,i],
                              alleleArray[levs,i,]
                              )
      else transTables[[i]] <- matrix(c(alleleMat[levs,i],alleleArray[levs,i,]),
                              1,3)
      colnames(transTables[[i]]) <- c("levels",
                                      paste("allele",1:ploidy,sep="."))
      
                              
    }
  names(transTables) <- markerNames


  callCodes <- matrix(as.integer(NA),
                      nrow=ncol(alleleMat),
                      ncol=nrow(alleleMat)
                      )
  rownames(callCodes) <- markerNames
  colnames(callCodes) <- rownames(data)

  for(i in 1:nMarker)
    {
      callCodes[i,] <- match(alleleMat[,i], transTables[[i]][,1])
    }
  
  retval <- list(
                 callCodes=callCodes,
                 transTables=transTables
                 )
  
  retval
  
}
